/******************************************************************************
    CallWave VoiceMail Widget

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

 ******************************************************************************/

/******************************************************************************
  VM_LoginView c'tor

    HTML DOM Dependencies:
        WIDGET_loginViewFrame           - <div>
        WIDGET_loginViewStatusTextArea  - <div>
        WIDGET_loginCellNumberTextArea  - <input type="text">
        WIDGET_loginPinTextArea         - <input type="password">

    Call backs:
        onAuthenticateSuccess( responseMessage )
        onGetAccountInfoSucceeded( responseMessage )
        onLoginRequired( statusMessage )

    Globals:
        VM_GetPreference( key )
        VM_SetPreference( key, value )
        EX_Log( message )
        VM_WIDGET_SOURCE
        VM_WIDGET_VERSION
        VM_WIDGET_OS_VERSION

 ******************************************************************************/
function VM_LoginView( onAuthenticateSuccess, onGetAccountInfoSucceeded, onLoginRequired )
{
    this.m_onAuthenticateSuccess        = onAuthenticateSuccess;
    this.m_onGetAccountInfoSucceeded    = onGetAccountInfoSucceeded;
    this.m_onLoginRequired              = onLoginRequired;

    this.m_loginRetryTimer = null;
    this.m_tryingAutologin = false;

    var self = this;
    this.m_accountHandler = new ACC_AccountHandler( VM_WIDGET_SOURCE,
                                                    VM_WIDGET_VERSION,
                                                    VM_WIDGET_OS_VERSION );

    this.m_accountHandler.m_onAuthenticateSucceeded   = function( responseMessage )
    {
        self.onAuthenticateSucceeded( responseMessage );
    };

    this.m_accountHandler.m_onAuthenticateFailed      = function( statusMessage  )
    {
        self.onAuthenticateFailed( statusMessage );
    };

    this.m_accountHandler.m_onForgotPinSucceeded      = function( responseMessage )
    {
        self.onForgotPinSucceeded( responseMessage );
    };

    this.m_accountHandler.m_onForgotPinFailed         = function( statusMessage )
    {
        self.onForgotPinFailed( statusMessage );
    };

    this.m_accountHandler.m_onGetAccountInfoSucceeded = function( responseMessage )
    {
        self.onGetAccountInfoSucceeded( responseMessage );
    };

    this.m_accountHandler.m_onGetAccountInfoFailed    = function( statusMessage )
    {
        self.onGetAccountInfoFailed( statusMessage );
    }

    function AddEditFieldMethod( self, identifier )
    {
        self['Set' + identifier + 'Data'] = function( message ) {
            $('#WIDGET_login' + identifier + 'TextArea').val(message);
        };

        self['Get' + identifier + 'Data'] = function() {
            return $('#WIDGET_login' + identifier + 'TextArea').val();
        }

        self['Get' + identifier + 'Obj'] = function() {
            return $('#WIDGET_login' + identifier + 'TextArea' )[0];
        }
    }

    AddEditFieldMethod( this, 'CellNumber' );
    AddEditFieldMethod( this, 'Pin' );
}
VM_LoginView.prototype =
{
    VM_DEFAULT_PIN_REQUEST_ERROR_MESSAGE : "Failed to send a PIN reminder. Please try again later.",
    VM_DEFAULT_LOGIN_ERROR_MESSAGE       : "Login failed. Please try again later.",
    LOGIN_RETRY_INTERVAL_SECONDS         : 20,

    enableRetryTimer:function( )
    {
        this.disableRetryTimer( );

        var self = this;
        var handleGetAccountInfo = function( )
        {
            self.handleGetAccountInfo( );
        };
        this.m_loginRetryTimer = setInterval( handleGetAccountInfo, this.LOGIN_RETRY_INTERVAL_SECONDS * 1000 );
    },

    disableRetryTimer:function( )
    {
        if ( this.m_loginRetryTimer != null )
        {
            clearInterval( this.m_loginRetryTimer );
            this.m_loginRetryTimer = null;
        }
    },

    SetStatusText : function ( message )
    {
        var msg = message
        var url = msg.match( "www[/./?/&/=a-zA-Z0-9]*[^\.]" );


        msg = url && url.length ? tag('a',
                                      { href: 'javascript:void(0)', onClick: "OpenURL( 'http://" + url + "' );" },
                                      msg) : msg;

        document.getElementById( 'WIDGET_loginViewStatusTextArea' ).innerHTML = msg;
        if (arguments.length > 1)
        {
            document.getElementById( 'WIDGET_loginViewStatusTextArea' ).style.color = arguments[1]
        }
        else
        {
            document.getElementById( 'WIDGET_loginViewStatusTextArea' ).style.color = "#bf0000"
        }
    },

    /**************************************************************************
      View repainting methods
     **************************************************************************/

    reset:function( )
    {
        document.getElementById( "WIDGET_loginViewFrame" ).style.display = "none";
        document.getElementById( "WIDGET_autoLoginViewFrame" ).style.display = "none";
    },

    refresh:function( )
    {
        if (this.m_tryingAutologin)
        {
            document.getElementById( "WIDGET_loginViewFrame" ).style.display = "none";
            document.getElementById( "WIDGET_autoLoginViewFrame" ).style.display = "block";
        }
        else
        {
            document.getElementById( "WIDGET_loginViewFrame" ).style.display = "block";
            document.getElementById( "WIDGET_autoLoginViewFrame" ).style.display = "none";
        }

        this.SetStatusText( "Please enter your registered phone number and PIN to continue.", "#000000" );

        var cellNumber = VM_GetPreference( PREF_KEY_CELL_NUMBER );

        if ( cellNumber )
        {
            this.SetCellNumberData( cellNumber );
        }

        if (window === top)
        {
            this.GetCellNumberObj( ).focus( );
        }
        else
        {
            if (this.GetCellNumberObj( ).setActive) this.GetCellNumberObj( ).setActive( );
        }
    },

    resetLoginForm: function()
    {
        this.m_tryingAutologin = false;
        document.getElementById("WIDGET_loginViewFrame").style.display = "block";
        document.getElementById("WIDGET_autoLoginViewFrame").style.display = "none";
        var btn = document.getElementById("WIDGET_loginSubmitButton");
        if (btn)
        {
            btn.disabled = false;
            btn.className = (btn.className) ? btn.className.replace(/\s*disabled$/, "") : btn.className;
        }
        if (window._IG_AdjustIFrameHeight) window._IG_AdjustIFrameHeight();
    },

    /**************************************************************************
      Authenticate methods
     **************************************************************************/

    handleAuthenticateSubmit:function( )
    {
        try
        {
            var cellNumber = UTILS_StripNonDigits( this.GetCellNumberData( ) );

            if ( cellNumber.length != 10 )
            {
                this.SetCellNumberData( "" );
                this.SetStatusText( "Please enter your 10-digit registered cell number." );
                return false;
            }

            var PIN = UTILS_StripNonDigits( this.GetPinData( ) );

            if ( PIN.length != this.GetPinData( ).length )
            {
                this.SetPinData( "" );
                this.SetStatusText( "Please enter your 4-10 digit PIN (numeric characters only)." );
                return false;
            }

            if ( PIN.length < 4 || PIN.length > 10  )
            {
                this.SetPinData( "" );
                this.SetStatusText( "Please enter your 4-10 digit PIN." );
                return false;
            }

            VM_SetPreference( PREF_KEY_CELL_NUMBER, cellNumber );

            this.SetStatusText( "Connecting, please wait...", "#000000" );

            var btn = document.getElementById("WIDGET_loginSubmitButton");
            if (btn)
            {
                btn.disabled = true;
                btn.className = (btn.className) ? btn.className + " disabled" : "disabled";
            }

            this.m_accountHandler.authenticate( cellNumber, PIN );

        }
        catch ( e )
        {
            this.SetStatusText( "Login failed. Please try again later." );

            EX_Log( "VM_LoginView::handleAuthenticateSubmit( ) : " + e );
        }
    },

    onAuthenticateSucceeded:function( responseMessage )
    {
        try
        {
            //alert( "VM_LoginView::onAuthenticateSucceeded: responseMessage.m_status is " + responseMessage.m_status );

            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                {
                    // Save persistent account attributes
                    VM_SetPreference( PREF_KEY_ACCOUNT, responseMessage.m_account );
                    VM_SetPreference( PREF_KEY_U_STRING, responseMessage.m_uString );

                    this.m_onAuthenticateSuccess( );

                    break;
                }

            case MSG_ResponseStatus.UNEXPECTED_ERROR:
                {
                    this.SetStatusText( VM_DEFAULT_LOGIN_ERROR_MESSAGE );

                    break;
                }

            case MSG_ResponseStatus.GENERAL_ERROR:
                {
                    this.SetStatusText( responseMessage.m_statusMsg );

                    break;
                }

            case MSG_ResponseStatus.RECOVERABLE_ERROR:
                {
                    this.SetStatusText( "Login failed. Please try again." );

                    break;
                }

            default:
                {
                    this.SetStatusText(
                        ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 ) ?
                            responseMessage.m_statusMsg :
                            this.VM_DEFAULT_LOGIN_ERROR_MESSAGE );

                    break;
                }
            }
        }
        catch ( e )
        {
            var theError = new CWError( 'CWError' );
            theError.Nest( e );

            EX_Log( "VM_LoginView::onAuthenticateSucceeded( ) : " + theError.message );
        }
        this.resetLoginForm();
    },

    onAuthenticateFailed:function( statusMessage )
    {
        try
        {
            this.SetStatusText( this.VM_DEFAULT_LOGIN_ERROR_MESSAGE );

            EX_Log( "VM_LoginView::onAuthenticateFailed( ) : " + statusMessage );
        }
        catch ( e )
        {
            EX_Log( "VM_LoginView::onAuthenticateFailed( ) : " + e.message );
        }

        this.resetLoginForm();
    },


    /**************************************************************************
      Forgot PIN methods
     **************************************************************************/

    handlePinReminderRequested:function( )
    {
        try
        {
            var cellNumber = UTILS_StripNonDigits( this.GetCellNumberData( ) );

            if ( cellNumber.length != 10 )
            {
                this.SetCellNumberData( "" );
                this.SetStatusText( "Please enter your 10-digit registered cell number." );
                return;
            }

            this.SetPinData( "" );

            VM_SetPreference( PREF_KEY_CELL_NUMBER, cellNumber );

            this.m_accountHandler.forgotPin( cellNumber );
        }
        catch ( e )
        {
            this.SetStatusText( this.VM_DEFAULT_PIN_REQUEST_ERROR_MESSAGE );

            EX_Log( "VM_LoginView::handlePinReminderRequested( ) : Failed to submit manual login : " + e );
        }
    },

    onForgotPinSucceeded:function( responseMessage )
    {
        try
        {
            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                {
                    this.SetStatusText( "PIN reminder sent to your cell phone. Enter your PIN when ready." );
                    break;
                }

            case MSG_ResponseStatus.UNEXPECTED_ERROR:
                {
                    this.SetStatusText( this.VM_DEFAULT_PIN_REQUEST_ERROR_MESSAGE );
                    break;
                }

            case MSG_ResponseStatus.GENERAL_ERROR:
                {
                    this.SetStatusText( responseMessage.m_statusMsg );

                    break;
                }

            case MSG_ResponseStatus.RECOVERABLE_ERROR:
                {
                    this.SetStatusText( "Failed to request a PIN reminder. Please try again." );

                    break;
                }

            default:
                {
                    this.SetStatusText(
                        ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 ) ?
                            responseMessage.m_statusMsg :
                            this.VM_DEFAULT_PIN_REQUEST_ERROR_MESSAGE );

                    break;
                }
            }
        }
        catch ( e )
        {
            this.SetStatusText( this.VM_DEFAULT_PIN_REQUEST_ERROR_MESSAGE );

            EX_Log( "VM_RegisterView::onForgotPinSucceeded( ) : " + e );
        }
    },

    onForgotPinFailed:function( statusMessage )
    {
        this.SetStatusText( this.VM_DEFAULT_PIN_REQUEST_ERROR_MESSAGE );

        EX_Log( "VM_LoginView::onForgotPinFailed( ) : " + statusMessage );
    },

    /**************************************************************************
      GetAccountInfo methods
     **************************************************************************/
    handleGetAccountInfo:function( )
    {
        this.m_tryingAutologin = (arguments.length > 0 && arguments[0]); // from autologin

        try
        {

            this.m_accountHandler.getAccountInfo( VM_GetPreference( PREF_KEY_ACCOUNT ),
                                                  VM_GetPreference( PREF_KEY_U_STRING ) );
        }
        catch ( e )
        {
            EX_Log( "VM_LoginView::handleGetAccountInfo( ) : Failed to get account info : " + e );

            this.enableRetryTimer( );
        }
    },

    onGetAccountInfoSucceeded:function( responseMessage )
    {
        try
        {
            //alert( "VM_LoginView::onGetAccountInfoSucceeded: responseMessage.m_status is " + responseMessage.m_status );

            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                // Success
                {
                    this.m_onGetAccountInfoSucceeded( responseMessage );
                    break;
                }

            case MSG_ResponseStatus.GENERAL_ERROR:
            case MSG_ResponseStatus.GETA_LOGIN_FAILURE:
            case MSG_ResponseStatus.GETA_UPGRADE_REQUIRED:
                {
                    // Disable retry timer
                    this.disableRetryTimer( );

                    // Go to the login screen
                    this.m_onLoginRequired(
                            ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 ) ?
                            responseMessage.m_statusMsg :
                            this.VM_DEFAULT_LOGIN_ERROR_MESSAGE );
                    this.resetLoginForm();

                    break;
                }
            default:
                // Other error
                {
                    // Retry
                    this.enableRetryTimer( );

                    break;
                }
            }
        }
        catch ( e )
        {
            this.enableRetryTimer( );

            EX_Log( "VM_LoginView::onGetAccountInfoSucceeded( ) : " + e.message );
        }
    },

    onGetAccountInfoFailed:function( statusMessage )
    {
        //alert( "VM_LoginView::onGetAccountInfoFailed" );

        // Retry
        this.enableRetryTimer( );
    }
}
